﻿// <copyright file="PTestViewModel.cs" company="Sirona Dental Systems GmbH">
//    Copyright (c) Sirona Dental Systems GmbH. All rights reserved.
// </copyright>

using System;
using System.IO;
using System.ComponentModel;
using System.Globalization;
using System.Threading;
using System.Windows;
using System.Windows.Input;
using Microsoft.Win32;

namespace PTest
{
    using Properties;

    internal class PTestViewModel : INotifyPropertyChanged
    {
        #region private fields

        private bool        _checkBoxFieldSizeLimit = true;
        private SlidaTokens _tokenSelection;
        private PTestModel  _pTestModel;

        #endregion

        public event PropertyChangedEventHandler PropertyChanged;

        #region commands

        public ICommand LoadData { get; private set; }
        public ICommand Send { get; private set; }
        public ICommand SaveData { get; private set; }
        public ICommand TaskSwitch { get; private set; }
        public ICommand CreateData { get; private set; }
        public ICommand InsertDateTime { get; private set; }
        public ICommand About { get; private set; }
        public ICommand ClearData { get; private set; }
        public ICommand Exit { get; private set; }
        public ICommand BrowseSlidaPath { get; private set; }
        public ICommand BrowseSidexisPath { get; private set; }
        public ICommand BrowseImageFilename { get; private set; }
        
        #endregion

        #region SlidaSizeLimits

        private const int SIZE_LIMIT_NAME = 32;
        private const int SIZE_LIMIT_DATE = 10;
        private const int SIZE_LIMIT_TIME = 8;
        private const int SIZE_LIMIT_INDEX = 20;
        private const int SIZE_LIMIT_SEX = 1;
        private const int SIZE_LIMIT_PREGNANCY = 1;
        private const int SIZE_LIMIT_DENTIST = 12;
        private const int SIZE_LIMIT_STATION = 20;
        private const int SIZE_LIMIT_TYPE = 4;
        private const int SIZE_LIMIT_RESPONSIBLE = 12;
        private const int SIZE_LIMIT_REASON = 31;
        private const int SIZE_LIMIT_DIAGNOSIS = 99;
        private const int SIZE_LIMIT_INTVALUE = 5;
        private const int SIZE_LIMIT_FILENAME = 99;

        #endregion

        #region TextBoxEnableFlags

        public bool TextBoxNameActive { get; set; }
        public bool TextBoxFirstNameActive { get; set; }
        public bool TextBoxDateOfBirthActive { get; set; }
        public bool TextBoxExtCardIndexNoActive { get; set; }
        public bool TextBoxNameNewActive { get; set; }
        public bool TextBoxFirstNameNewActive { get; set; }
        public bool TextBoxDateOfBirthNewActive { get; set; }
        public bool TextBoxExtCardIndexNoNewActive { get; set; }
        public bool TextBoxSexNewActive { get; set; }
        public bool TextBoxPregnancyNewActive { get; set; }
        public bool TextBoxPermanentDentistNewActive { get; set; }
        public bool TextBoxComputernameActive { get; set; }
        public bool DatePickerOrderDateActive { get; set; }
        public bool TextBoxOrderTimeActive { get; set; }
        public bool TextBoxSenderActive { get; set; }
        public bool TextBoxReceiverActive { get; set; }
        public bool TextBoxOrderNoActive { get; set; }
        public bool TextBoxImageNoActive { get; set; }
        public bool TextBoxImageTypeActive { get; set; }
        public bool TextBoxDateOfImageActive { get; set; }
        public bool TextBoxTimeOfImageActive { get; set; }
        public bool TextBoxResponsiblePersonActive { get; set; }
        public bool TextBoxReasonActive { get; set; }
        public bool TextBoxDiagnosisActive { get; set; }
        public bool TextBoxRadiationTimeActive { get; set; }
        public bool TextBoxTubeVoltageActive { get; set; }
        public bool TextBoxTubeCurrentActive { get; set; }
        public bool TextBoxFilenameActive { get; set; }

        #endregion

        #region Constructors

        public PTestViewModel()
        {
            // Zeitformat unabhängig von den Systemeinstellungen auf Slida festlegen.
            var ci = CultureInfo.CreateSpecificCulture(CultureInfo.CurrentCulture.Name);

            ci.DateTimeFormat.ShortDatePattern = "dd.MM.yyyy";
            ci.DateTimeFormat.ShortTimePattern = "HH:mm:ss";

            Thread.CurrentThread.CurrentCulture = ci;

            LoadData = new ActionCommand(OnLoadDataExecuted, x => true);
            Send = new ActionCommand(OnSendExecuted, x => true);
            SaveData = new ActionCommand(OnSaveDataExecuted, x => true);
            TaskSwitch = new ActionCommand(OnTaskSwitchExecuted, x => true);
            CreateData = new ActionCommand(OnCreateDataExecuted, x => true);
            InsertDateTime = new ActionCommand(OnInsertDateTimeExecuted, x => true);
            About = new ActionCommand(OnAboutExecuted, x => true);
            ClearData = new ActionCommand(OnClearDataExecuted, x => true);
            Exit = new ActionCommand(OnExitExecuted, x => true);

            BrowseSlidaPath = new ActionCommand(OnBrowseSlidaPath, x => true);
            BrowseSidexisPath = new ActionCommand(OnBrowseSidexisPath, x => true);
            BrowseImageFilename = new ActionCommand(OnBrowseImageFilename, x => true);

            _pTestModel = ((App) Application.Current).PTestData;
            TokenSelection = SlidaTokens.A;
        }
        
        #endregion

        #region globalData

        public SlidaTokens TokenSelection
        {
            get { return _tokenSelection; }
            
            set
            {
                _tokenSelection = value;

                TextBoxSenderActive = true;
                TextBoxReceiverActive = true;

                switch (value)
                {
                    case SlidaTokens.A:
                        EnablePatientRelatedData();
                        EnablePatientRelatedDataNew(false);
                        EnablePracticeRelevantData();
                        EnableImageRelatedData(false);
                        TextBoxImageNoActive = true;
                        break;

                    case SlidaTokens.I:
                        EnablePatientRelatedData();
                        EnablePatientRelatedDataNew(false);
                        EnablePracticeRelevantData(false);
                        EnableImageRelatedData();
                        TextBoxOrderNoActive = false;
                        TextBoxFilenameActive = false;
                        break;

                    case SlidaTokens.M:
                        EnablePatientRelatedData();
                        EnablePatientRelatedDataNew(false);
                        EnablePracticeRelevantData(false);
                        EnableImageRelatedData(false);
                        TextBoxImageNoActive = true;
                        TextBoxImageTypeActive = true;
                        TextBoxDateOfImageActive = true;
                        TextBoxTimeOfImageActive = true;
                        TextBoxResponsiblePersonActive = true;
                        TextBoxFilenameActive = true;
                        break;

                    case SlidaTokens.N:
                        EnablePatientRelatedData(false);
                        EnablePatientRelatedDataNew();
                        EnablePracticeRelevantData(false);
                        EnableImageRelatedData(false);
                        TextBoxPregnancyNewActive = false;
                        break;

                    case SlidaTokens.S:
                        EnablePatientRelatedData();
                        EnablePatientRelatedDataNew(false);
                        EnablePracticeRelevantData();
                        EnableImageRelatedData(false);
                        break;

                    case SlidaTokens.T:
                        EnablePatientRelatedData();
                        EnablePatientRelatedDataNew(false);
                        TextBoxSexNewActive = true;
                        TextBoxPregnancyNewActive = true;
                        EnablePracticeRelevantData(false);
                        EnableImageRelatedData();
                        TextBoxFilenameActive = false;
                        break;

                    case SlidaTokens.U:
                        EnablePatientRelatedData();
                        EnablePatientRelatedDataNew();
                        TextBoxPregnancyNewActive = false;
                        EnablePracticeRelevantData(false);
                        EnableImageRelatedData(false);
                        break;

                    case SlidaTokens.X:
                        EnablePatientRelatedData();
                        EnablePatientRelatedDataNew(false);
                        TextBoxSexNewActive = true;
                        TextBoxPregnancyNewActive = true;
                        EnablePracticeRelevantData();
                        EnableImageRelatedData(false);
                        TextBoxOrderNoActive = true;
                        TextBoxImageTypeActive = true;
                        TextBoxReasonActive = true;
                        break;
                }
                OnPropertyChanged();
            }
        }

        public string TextBoxMailslot
        {
            get { return ((App)Application.Current).slidaFile; }

            set
            {
                ((App)Application.Current).slidaFile = value;
                OnPropertyChanged("TextBoxMailslot");
            }
        }

        public string TextBoxTaskSwitch
        {
            get { return ((App)Application.Current).sidexisFile; }

            set
            {
                ((App)Application.Current).sidexisFile = value;
                OnPropertyChanged("TextBoxTaskSwitch");
            }
        }

        public bool CheckBoxFieldSizeLimit
        {
            get { return _checkBoxFieldSizeLimit; }

            set
            {
                _checkBoxFieldSizeLimit = value;
                OnPropertyChanged(); // Force all field to be trimmed
            }
        }

        #endregion

        #region PatientRelevantData

        public string TextBoxName
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.Name, SIZE_LIMIT_NAME); }

            set
            {
                _pTestModel.Name = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_NAME);
                OnPropertyChanged("TextBoxName");
            }
        }


        public string TextBoxFirstName
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.FirstName, SIZE_LIMIT_NAME); }

            set
            {
                _pTestModel.FirstName = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_NAME);
                OnPropertyChanged("TextBoxFirstName");
            }
        }

        public string TextBoxDateOfBirth
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.DateOfBirth, SIZE_LIMIT_DATE); }

            set
            {
                _pTestModel.DateOfBirth = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_DATE);
                OnPropertyChanged("TextBoxDateOfBirth");
            }
        }


        public string TextBoxExtCardIndexNo
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.ExtCardIndexNo, SIZE_LIMIT_INDEX); }

            set
            {
                _pTestModel.ExtCardIndexNo = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_INDEX);
                OnPropertyChanged("TextBoxExtCardIndexNo");
            }
        }

        #endregion

        #region PatientRelevantDataNew

        public string TextBoxNameNew
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.NameNew, SIZE_LIMIT_NAME); }

            set
            {
                _pTestModel.NameNew = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_NAME);
                OnPropertyChanged("TextBoxNameNew");
            }
        }

        public string TextBoxFirstNameNew
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.FirstNameNew, SIZE_LIMIT_NAME); }

            set
            {
                _pTestModel.FirstNameNew = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_NAME);
                OnPropertyChanged("TextBoxFirstNameNew");
            }
        }

        public string TextBoxDateOfBirthNew
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.DateOfBirthNew, SIZE_LIMIT_DATE); }

            set
            {
                _pTestModel.DateOfBirthNew = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_DATE);
                OnPropertyChanged("TextBoxDateOfBirthNew");
            }
        }

        public string TextBoxExtCardIndexNoNew
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.ExtCardIndexNoNew, SIZE_LIMIT_INDEX); }

            set
            {
                _pTestModel.ExtCardIndexNoNew = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_INDEX);
                OnPropertyChanged("TextBoxExtCardIndexNoNew");
            }
        }

        public string TextBoxSexNew
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.SexNew, SIZE_LIMIT_SEX); }

            set
            {
                _pTestModel.SexNew = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_SEX);
                OnPropertyChanged("TextBoxSexNew");
            }
        }

        public string TextBoxPregnancyNew
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.PregnancyNew, SIZE_LIMIT_PREGNANCY); }

            set
            {
                _pTestModel.PregnancyNew = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_PREGNANCY);
                OnPropertyChanged("TextBoxPregnancyNew");
            }
        }

        public string TextBoxPermanentDentistNew
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.PermanentDentistNew, SIZE_LIMIT_DENTIST); }

            set
            {
                _pTestModel.PermanentDentistNew = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_DENTIST);
                OnPropertyChanged("TextBoxPermanentDentistNew");
            }
        }

        #endregion

        #region PracticeRelevantData

        public string TextBoxComputername
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.Computername, SIZE_LIMIT_STATION); }

            set
            {
                _pTestModel.Computername = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_STATION);
                OnPropertyChanged("TextBoxComputername");
            }
        }

        public DateTime? DatePickerOrderDate
        {
            get
            {
                if (string.IsNullOrEmpty(_pTestModel.OrderDate))
                    return null;

                DateTime dateTime;

                if (DateTime.TryParseExact( _pTestModel.OrderDate, "dd.MM.yyyy", CultureInfo.InvariantCulture, DateTimeStyles.None, out dateTime))
                    return dateTime;

                return null;
            }

            set
            {
                if (value.HasValue)
                    _pTestModel.OrderDate = value.Value.ToString("dd.MM.yyyy");
                else
                    _pTestModel.OrderDate = string.Empty;

                OnPropertyChanged("DatePickerOrderDate");
            }
        }

        public string TextBoxOrderTime
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.OrderTime, SIZE_LIMIT_TIME); }

            set
            {
                _pTestModel.OrderTime = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_TIME);
                OnPropertyChanged("TextBoxOrderTime");
            }
        }

        public string TextBoxSender
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.Sender, SIZE_LIMIT_STATION); }

            set
            {
                _pTestModel.Sender = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_STATION);
                OnPropertyChanged("TextBoxSender");
            }
        }

        public string TextBoxReceiver
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.Receiver, SIZE_LIMIT_STATION); }

            set
            {
                _pTestModel.Receiver = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_STATION);
                OnPropertyChanged("TextBoxReceiver");
            }
        }

        #endregion

        #region ImageRelatedData

        public string TextBoxOrderNo
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.OrderNo, SIZE_LIMIT_INDEX); }

            set
            {
                CheckIfInt(value);
                _pTestModel.OrderNo = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_INDEX);
                OnPropertyChanged("TextBoxOrderNo");
            }
        }
        
        public string TextBoxImageNo
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.ImageNo, SIZE_LIMIT_INDEX); }

            set
            {
                CheckIfInt(value);
                _pTestModel.ImageNo = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_INDEX);
                OnPropertyChanged("TextBoxImageNo");
            }
        }

        public string TextBoxImageType
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.ImageType, SIZE_LIMIT_TYPE); }

            set
            {
                _pTestModel.ImageType = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_TYPE);
                OnPropertyChanged("TextBoxImageType");
            }
        }

        public string TextBoxDateOfImage
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.DateOfImage, SIZE_LIMIT_DATE); }

            set
            {
                _pTestModel.DateOfImage = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_DATE);
                OnPropertyChanged("TextBoxDateOfImage");
            }
        }

        public string TextBoxTimeOfImage
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.TimeOfImage, SIZE_LIMIT_TIME); }

            set
            {
                _pTestModel.TimeOfImage = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_TIME);
                OnPropertyChanged("TextBoxTimeOfImage");
            }
        }

        public string TextBoxResponsiblePerson
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.ResponsiblePerson, SIZE_LIMIT_RESPONSIBLE); }

            set
            {
                _pTestModel.ResponsiblePerson = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_RESPONSIBLE);
                OnPropertyChanged("TextBoxResponsiblePerson");
            }
        }

        public string TextBoxReason
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.Reason, SIZE_LIMIT_REASON); }

            set
            {
                _pTestModel.Reason = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_REASON);
                OnPropertyChanged("TextBoxReason");
            }
        }

        public string TextBoxDiagnosis
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.Diagnosis, SIZE_LIMIT_DIAGNOSIS); }

            set
            {
                _pTestModel.Diagnosis = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_DIAGNOSIS);
                OnPropertyChanged("TextBoxDiagnosis");
            }
        }

        public string TextBoxRadiationTime
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.RadiationTime, SIZE_LIMIT_INTVALUE); }

            set
            {
                _pTestModel.RadiationTime = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_INTVALUE);
                OnPropertyChanged("TextBoxRadiationTime");
            }
        }

        public string TextBoxTubeVoltage
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.TubeVoltage, SIZE_LIMIT_INTVALUE); }

            set
            {
                _pTestModel.TubeVoltage = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_INTVALUE);
                OnPropertyChanged("TextBoxTubeVoltage");
            }
        }

        public string TextBoxTubeCurrent
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.TubeCurrent, SIZE_LIMIT_INTVALUE); }

            set
            {
                _pTestModel.TubeCurrent = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_INTVALUE);
                OnPropertyChanged("TextBoxTubeCurrent");
            }
        }

        public string TextBoxFilename
        {
            get { return CutToTextFieldLimitIfLimitOn(_pTestModel.Filename, SIZE_LIMIT_FILENAME); }

            set
            {
                _pTestModel.Filename = CutToTextFieldLimitIfLimitOn(value, SIZE_LIMIT_FILENAME);
                OnPropertyChanged("TextBoxFilename");
            }
        }

        #endregion

        #region privat Functions

        private string CutToTextFieldLimitIfLimitOn(string text, int maxTextSize)
        {
            return _checkBoxFieldSizeLimit ? text.Substring(0, Math.Min(text.Length, maxTextSize)) : text;
        }

        private void EnableImageRelatedData(bool enable = true)
        {
            TextBoxOrderNoActive = enable;
            TextBoxImageNoActive = enable;
            TextBoxImageTypeActive = enable;
            TextBoxDateOfImageActive = enable;
            TextBoxTimeOfImageActive = enable;
            TextBoxResponsiblePersonActive = enable;
            TextBoxReasonActive = enable;
            TextBoxDiagnosisActive = enable;
            TextBoxRadiationTimeActive = enable;
            TextBoxTubeVoltageActive = enable;
            TextBoxTubeCurrentActive = enable;
            TextBoxFilenameActive = enable;
        }

        private void EnablePatientRelatedData(bool enable = true)
        {
            TextBoxNameActive = enable;
            TextBoxFirstNameActive = enable;
            TextBoxDateOfBirthActive = enable;
            TextBoxExtCardIndexNoActive = enable;
        }

        private void EnablePatientRelatedDataNew(bool enable = true)
        {
            TextBoxNameNewActive = enable;
            TextBoxFirstNameNewActive = enable;
            TextBoxDateOfBirthNewActive = enable;
            TextBoxExtCardIndexNoNewActive = enable;
            TextBoxSexNewActive = enable;
            TextBoxPregnancyNewActive = enable;
            TextBoxPermanentDentistNewActive = enable;
        }

        private void EnablePracticeRelevantData(bool enable = true)
        {
            TextBoxComputernameActive = enable;
            DatePickerOrderDateActive = enable;
            TextBoxOrderTimeActive = enable;
        }

        private void OnPropertyChanged(string propertyName = null)
        {
            if (PropertyChanged != null)
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        private static void CheckIfInt(string value)
        {
            if (value == string.Empty)
                return;

            int num;

            if (int.TryParse(value, out num) == false)
                throw new ArgumentOutOfRangeException("Should be a number");
        }

        #endregion

        #region command handler

        private void OnClearDataExecuted(object obj)
        {
            _pTestModel.ClearData();
            OnPropertyChanged();
        }

        private void OnAboutExecuted(object obj)
        {
            var about = new About();
            about.ShowDialog();
        }

        private void OnCreateDataExecuted(object parameter)
        {
            ((App) Application.Current).PTestData.CreateData();
            OnPropertyChanged();
        }

        private void OnInsertDateTimeExecuted(object obj)
        {
            DatePickerOrderDate = DateTime.Now;
            TextBoxOrderTime = DateTime.Now.ToString("HH:mm:ss");
        }

        private void OnTaskSwitchExecuted(object obj)
        {
            _pTestModel.TaskSwitch();
        }

        private void OnSaveDataExecuted(object obj)
        {
            _pTestModel.SaveData();
        }

        private void OnSendExecuted(object obj)
        {
            _pTestModel.SendData(TextBoxMailslot, _tokenSelection);
        }

        private void OnLoadDataExecuted(object obj)
        {
            _pTestModel.LoadData();
            _pTestModel = ((App) Application.Current).PTestData;
            // the XML serialize create a new object so we must update it in our class
            OnPropertyChanged();
        }

        private void OnExitExecuted(object parameter)
        {
            Application.Current.Shutdown();
        }

        private void OnBrowseSidexisPath(object obj)
        {
            var openFileDialog = new OpenFileDialog();

            openFileDialog.Title = Resources.BrowseSidexisPath_Title;
            openFileDialog.Filter = Resources.BrowseSidexisPath_Filter;
            openFileDialog.InitialDirectory = TryToGetDir(TextBoxTaskSwitch);
            openFileDialog.FileName = TryToGetFileName(TextBoxTaskSwitch);

            if (openFileDialog.ShowDialog() == true)
                TextBoxTaskSwitch = openFileDialog.FileName;
        }

        private void OnBrowseSlidaPath(object obj)
        {
            var openFileDialog = new OpenFileDialog();

            openFileDialog.Title = Resources.BrowseSlidaPath_Title;
            openFileDialog.Filter = Resources.BrowseSlidaPath_Filter;
            openFileDialog.InitialDirectory = TryToGetDir(TextBoxMailslot);
            openFileDialog.FileName = TryToGetFileName(TextBoxMailslot);

            if (openFileDialog.ShowDialog() == true)
                TextBoxMailslot = openFileDialog.FileName;
        }

        private void OnBrowseImageFilename(object obj)
        {
            var openFileDialog = new OpenFileDialog();

            openFileDialog.Title = Resources.BrowseImageFilename_Title;
            openFileDialog.Filter = Resources.BrowseImageFilename_Filter;

            var slidaDir = TryToGetDir(TextBoxMailslot);

            if (string.IsNullOrEmpty(slidaDir))
            {
                MessageBoxEx.Show( Resources.ApplicationName, Resources.BrowseImageFilename_EmptySlidaMailslot, MessageBoxButton.OK, MessageBoxExImage.Information );
                return;
            }
            openFileDialog.InitialDirectory = slidaDir;

            if (openFileDialog.ShowDialog() == true)
            {
                var selectedFile = openFileDialog.FileName;

                if (selectedFile.StartsWith(slidaDir) == false)
                    MessageBoxEx.Show( Resources.ApplicationName, Resources.ErrorImageFilenamePlace, MessageBoxButton.OK, MessageBoxExImage.Error );
                else
                    TextBoxFilename = selectedFile.Substring(slidaDir.Length + 1);
            }
        }

        private string TryToGetDir(string path)
        {
            if (string.IsNullOrEmpty(path))
                return null;

            try
            {
                return Path.GetDirectoryName(path);
            }
            catch (Exception)
            {
                return null;
            }
        }

        private string TryToGetFileName(string path)
        {
            if (string.IsNullOrEmpty(path))
                return null;

            try
            {
                return Path.GetFileName(path);
            }
            catch (Exception)
            {
                return null;
            }
        }

        #endregion
    }
}